# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

PATTERN = r"BADGIE\s+TIME"

PATTERN_START = r"<!--\s+" + PATTERN + r"\s+-->"

PATTERN_END = r"<!--\s+END\s+" + PATTERN + r"\s+-->"



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

from .. import tokens as to
from ..models import Badge
from ._base import register_badges


def brettops_badge(name):
    return Badge(
        name=f"brettops-{name}",
        description=f"Show that this {name} project is a BrettOps project.",
        example=f"https://img.shields.io/badge/brettops-{name}-209cdf?labelColor=162d50",
        title=f"brettops {name}",
        link="https://brettops.io",
        image=f"https://img.shields.io/badge/brettops-{name}-209cdf?labelColor=162d50",
        weight=-100,
    )


register_badges(
    {
        to.BRETTOPS_CONTAINER: brettops_badge("container"),
        to.BRETTOPS_PACKAGE: brettops_badge("package"),
        to.BRETTOPS_PIPELINE: brettops_badge("pipeline"),
        to.BRETTOPS_ROLE: brettops_badge("role"),
        to.BRETTOPS_TOOL: brettops_badge("tool"),
    }
)



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

import yaml

from .. import tokens as to
from ..models import Context, Hook, HookMatch

HOOKS = {
    HookMatch(
        repo="https://github.com/pre-commit/mirrors-prettier/", hook="prettier"
    ): {to.PRETTIER},
    HookMatch(repo="https://github.com/psf/black/", hook="black"): {to.PYTHON_BLACK},
    HookMatch(repo="https://github.com/PyCQA/bandit/", hook="bandit"): {
        to.PYTHON_BANDIT
    },
    HookMatch(repo="https://github.com/PyCQA/isort/", hook="isort"): {to.PYTHON_ISORT},
    HookMatch(repo="https://github.com/PyCQA/docformatter/", hook="docformatter"): {
        to.PYTHON_DOCFORMATTER
    },
    HookMatch(
        repo="https://github.com/PyCQA/docformatter/", hook="docformatter-venv"
    ): {to.PYTHON_DOCFORMATTER},
    HookMatch(repo="https://github.com/pre-commit/mirrors-mypy/", hook="mypy"): {
        to.PYTHON_MYPY
    },
    HookMatch(
        repo="https://gitlab.com/buildgarden/tools/cici-tools/", hook="cici-update"
    ): {to.CICI_TOOLS},
}


def normalize_url(url: str):
    url = url.strip()
    if not url.endswith("/"):
        url += "/"
    return url


def match_hook(repo, hook):
    entry = HookMatch(repo=normalize_url(repo), hook=hook.strip())
    if entry in HOOKS:
        return Hook(tokens=HOOKS[entry], repo=entry.repo, hook=entry.hook)


def run(context: Context) -> list[Hook]:
    pre_commit_config = context.nodes[to.PRE_COMMIT_CONFIG][0]
    data = yaml.safe_load(open(pre_commit_config.path))
    nodes = []
    for repo in data["repos"]:
        for hook in repo["hooks"]:
            match = match_hook(repo["repo"], hook["id"])
            if match:
                nodes.append(match)
    return nodes



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

import os
from contextlib import contextmanager
from pathlib import Path
from urllib.parse import urlencode, urlparse


def add_to_query(url: str, params: dict[str, str]):
    parsed = urlparse(url)

    _params = {}
    for param in parsed.query.split("&"):
        try:
            key, value = param.split("=")
            _params[key] = value
        except ValueError:
            pass

    _params.update(params)

    query = urlencode(_params)
    return f"{parsed.scheme}://{parsed.netloc}{parsed.path}?{query}"


@contextmanager
def change_directory(path: Path):
    origin = Path().absolute()
    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(origin)



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

import json
import os
import urllib.error
import urllib.request
from pathlib import Path

import gitlab

from .. import tokens as to
from ..models import Context, GitLabProject, Project

GITLAB_URL = "https://gitlab.com"
API_V4_URL = f"{GITLAB_URL}/api/v4"

gl = gitlab.Gitlab(private_token=os.environ.get("GITLAB_PRIVATE_TOKEN", None))

HEADERS: dict[str, str] = {}
private_token = os.environ.get("GITLAB_PRIVATE_TOKEN")
if private_token:
    HEADERS.update({"PRIVATE-TOKEN": private_token})


def get_project(remote_path):
    glproject = gl.projects.get(remote_path)

    project = Project(
        local_path=Path.cwd(),
        url=glproject.web_url,
        ref=glproject.default_branch,
        name=glproject.name,
        namespace=glproject.namespace,
        full_name=glproject.name_with_namespace,
        path=glproject.path,
        full_path=glproject.path_with_namespace,
    )

    return glproject, project


def get_latest_pipeline(glproject):
    try:
        return glproject.pipelines.get("latest", ref=glproject.default_branch)
    except gitlab.exceptions.GitlabGetError:
        return None


def get_latest_release(project_id):
    try:
        url = f"{API_V4_URL}/projects/{project_id}/releases/permalink/latest"
        request = urllib.request.Request(url=url, headers=HEADERS)
        response = urllib.request.urlopen(request)
        content = response.read()
        return json.loads(content)
    except urllib.error.HTTPError:
        return None


def run(context: Context) -> list[GitLabProject]:
    remote = context.nodes[to.GITLAB][0]
    glproject, project = get_project(remote.path)
    gltokens = set()

    # get gitlab latest release badge
    release = get_latest_release(glproject.id)
    if release:
        gltokens.add(to.GITLAB_RELEASE)

    # add gitlab latest pipeline badge
    glpipeline = get_latest_pipeline(glproject)
    if glpipeline is not None:
        gltokens.add(to.GITLAB_PIPELINE)

        # add gitlab coverage badge
        if glpipeline.coverage is not None:
            gltokens.add(to.GITLAB_COVERAGE)

    gitlabproject = GitLabProject(
        tokens=gltokens,
        url=project.url,
        ref=project.ref,
        namespace=project.namespace,
        name=project.name,
        path=project.path,
        full_name=project.full_name,
        full_path=project.full_path,
    )

    return [gitlabproject]



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

from .. import tokens as to
from ..models import Badge
from ._base import register_badges

register_badges(
    {
        to.CICI_TOOLS: Badge(
            name="cici-tools",
            description="This project uses [cici-tools](https://gitlab.com/buildgarden/tools/cici-tools).",
            example="https://img.shields.io/badge/%E2%9A%A1_cici--tools-enabled-c0ff33",
            title="cici-tools enabled",
            link="https://gitlab.com/buildgarden/tools/cici-tools",
            image="https://img.shields.io/badge/%E2%9A%A1_cici--tools-enabled-c0ff33",
            weight=20,
        ),
    }
)



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

_BADGES = {}


def register_badge(klass):
    assert klass.name not in _BADGES
    _BADGES[klass.name] = klass
    return klass


def register_badges(badges):
    for token, badge in badges.items():
        assert token not in _BADGES
        _BADGES[token] = badge


def get_badge(token):
    assert _BADGES[token]



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

import re
from pathlib import Path
from typing import Optional

from .. import tokens as to
from ..models import Context, File
from ..project import get_project_paths

FILES = {
    ".gitignore": {to.GIT},
    ".gitmodules": {to.GIT},
    ".gitattributes": {to.GIT},
    ".pre-commit-config.ya?ml": {to.PRE_COMMIT_CONFIG},
    ".pre-commit-hooks.ya?ml": {to.PRE_COMMIT_HOOKS},
    ".secrets.baseline": {to.DETECT_SECRETS},
    ".gitlab-ci.yml": {to.GITLAB_CI_FILE},
    "setup.cfg": {to.PYTHON_SETUP_CFG},
    "setup.py": {to.PYTHON_SETUPTOOLS},
    "pyproject.toml": {to.PYTHON_PYPROJECT_TOML},
    "meta/main.ya?ml": {to.ANSIBLE_GALAXY},
}

RE_FILES = {
    re.compile(r"^" + pattern + r"$", re.IGNORECASE): tokens
    for pattern, tokens in FILES.items()
}


def match_file(path: Path) -> Optional[File]:
    for regex, tokens in RE_FILES.items():
        match = regex.match(str(path))
        if match:
            return File(tokens=tokens, path=path, pattern=match.re.pattern)


def run(_context: Context) -> list[File]:
    nodes = []
    for path in get_project_paths():
        file = match_file(path)
        if file:
            nodes.append(file)
    return nodes



# SPDX-FileCopyrightText: 2024 UL Research Institutes
# SPDX-License-Identifier: MIT

import re
from typing import Optional

from attrs import define

from .constants import PATTERN_END, PATTERN_START


@define
class Token:
    value: str
    line: int
    column: int
    type: Optional[str] = None


def tokenize(text: str):
    token_specification = [
        ("BLOCK", r"```"),
        ("START", PATTERN_START),
        ("END", PATTERN_END),
        ("TEXT", r"."),
        ("NEWLINE", r"\n"),
    ]
    tok_regex = "|".join("(?P<%s>%s)" % pair for pair in token_specification)
    line_num = 1
    line_start = 0
    for mo in re.finditer(tok_regex, text):
        kind = mo.lastgroup
        value = mo.group()
        column = mo.start() - line_start
        yield Token(type=kind, value=value, line=line_num, column=column)


def parse_text(text: str, badge_text: str = ""):
    tokens = list(tokenize(text))
    output = ""
    in_block = False
    while tokens:
        token = tokens.pop(0)
        if token.type == "START":
            output += token.value
            if not in_block:
                while token.type != "END":
                    token = tokens.pop(0)
                output += f"\n\n{badge_text}\n\n"
                output += token.value

        # added this only to support documenting the feature
        elif token.type == "BLOCK":
            output += token.value
            in_block = not in_block

        elif token.type == "NEWLINE":
            output += "\n"
        else:
            output += token.value
    return output
